@@ -1,5 +1,8 @@
 package LWP::Authen::OAuth2;
 
+# ABSTRACT: Make requests to OAuth2 APIs.
+our $VERSION = '0.20'; # VERSION
+
 use 5.006;
 use strict;
 use warnings;
@@ -216,9 +219,12 @@ sub refresh_access_token {
             if (not (ref($tokens))) {
                 # Did I get JSON?
                 my $data = eval {decode_json($tokens)};
+
                 if ($data and not $@) {
-                    # Assume I got it.
-                    $tokens = $data;
+                    my $class = $data->{_class} or croak("No _class in token_string '$tokens'");
+                    eval {load($class)};
+                    if ($@) { croak("Can't load access token class '$class': $@"); }
+                    $tokens = $class->from_ref($data);
                 }
             }
             return $self->_set_tokens(tokens => $tokens, skip_save => 1);
@@ -325,18 +331,22 @@ sub user_agent {
     return $self->{user_agent} ||= LWP::UserAgent->new();
 }
 
+
+1; # End of LWP::Authen::OAuth2
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 LWP::Authen::OAuth2 - Make requests to OAuth2 APIs.
 
 =head1 VERSION
 
-Version 0.15
-
-=cut
-
-our $VERSION = '0.16';
-
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -385,6 +395,12 @@ L<LWP::Authen::OAuth2::ServiceProvider::Strava>
 
 implemented by L<Leon Wright|https://github.com/techman83>
 
+=item * Withings
+
+L<LWP::Authen::OAuth2::ServiceProvider::Withings>
+
+implemented by L<Brian Foley|https://github.com/foleybri>
+
 =item * Yahoo
 
 L<LWP::Authen::OAuth2::ServiceProvider::Yahoo>
@@ -649,7 +665,7 @@ tokens too quickly, so this can be important.
 =item C<user_agent =E<gt> $ua,>
 
 What user agent gets used under the hood?  Defaults to a new
-L<lWP::UserAgent> created on the fly.
+L<LWP::UserAgent> created on the fly.
 
 =back
 
@@ -793,12 +809,6 @@ L<LWP::UserAgent> responds to.
 Get the user agent.  The default if none was explicitly set is a new
 L<LWP::UserAgent> object.
 
-=head1 AUTHOR
-
-Ben Tilly, C<< <btilly at gmail.com> >>
-
-currently maintained by Thomas Klausner, C<< <domm@cpan.org> >>
-
 =head2 Contributors
 
 =over
@@ -807,7 +817,7 @@ currently maintained by Thomas Klausner, C<< <domm@cpan.org> >>
 
 =item * L<Thomas Klausner|https://github.com/domm>
 
-=item * L<Alexander Dutton|https://github.com/alexsdutton>
+=item * L<Alex Dutton|https://github.com/alexdutton>
 
 =item * L<Chris|https://github.com/TheWatcher>
 
@@ -815,46 +825,7 @@ currently maintained by Thomas Klausner, C<< <domm@cpan.org> >>
 
 =item * L<Adam Millerchip|https://github.com/amillerchip>
 
-=back
-
-=head1 BUGS
-
-Please report any bugs or feature requests to
-C<bug-lwp-authen-oauth2 at rt.cpan.org>, or through
-the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=LWP-Authen-OAuth2>.  I will
-be notified, and then you'll automatically be notified of progress on your
-bug as I make changes.
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc LWP::Authen::OAuth2
-
-You can also look for information at:
-
-=over 4
-
-=item Github (submit patches here)
-
-L<https://github.com/domm/perl-oauth2>
-
-=item RT: CPAN's request tracker (report bugs here)
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=LWP-Authen-OAuth2>
-
-=item AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/LWP-Authen-OAuth2>
-
-=item CPAN Ratings
-
-L<http://cpanratings.perl.org/d/LWP-Authen-OAuth2>
-
-=item MetaCPAN
-
-L<https://metacpan.org/release/LWP-Authen-OAuth2>
+=item * L<André Brás|https://github.com/whity>
 
 =back
 
@@ -876,7 +847,7 @@ L<Thomas Klausner|https://github.com/domm> for reporting that client
 type specific parameters were not available when the client type was properly
 specified
 
-=item * L<Alexander Dutton|https://github.com/alexsdutton> for making
+=item * L<Alex Dutton|https://github.com/alexdutton> for making
 C<ServiceProvider> work without requiring subclassing.
 
 =item * L<Leon Wright|https://github.com/techman83> for adding a L<Strava | http://strava.com> Service Provider and various bug fixes
@@ -891,47 +862,25 @@ C<ServiceProvider> work without requiring subclassing.
 
 =back
 
-=head1 LICENSE AND COPYRIGHT
+=head1 AUTHORS
 
-Copyright 2013 Rent.com.
+=over 4
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of the the Artistic License (2.0). You may obtain a
-copy of the full license at:
+=item *
 
-L<http://www.perlfoundation.org/artistic_license_2_0>
+Ben Tilly, <btilly at gmail.com>
 
-Any use, modification, and distribution of the Standard or Modified
-Versions is governed by this Artistic License. By using, modifying or
-distributing the Package, you accept this license. Do not use, modify,
-or distribute the Package, if you do not accept this license.
+=item *
 
-If your Modified Version has been derived from a Modified Version made
-by someone other than you, you are nevertheless required to ensure that
-your Modified Version complies with the requirements of this license.
+Thomas Klausner <domm@plix.at>
 
-This license does not grant you the right to use any trademark, service
-mark, tradename, or logo of the Copyright Holder.
+=back
 
-This license includes the non-exclusive, worldwide, free-of-charge
-patent license to make, have made, use, offer to sell, sell, import and
-otherwise transfer the Package with respect to any patent claims
-licensable by the Copyright Holder that are necessarily infringed by the
-Package. If you institute patent litigation (including a cross-claim or
-counterclaim) against any party alleging that the Package constitutes
-direct or contributory patent infringement, then this Artistic License
-to you shall terminate on the date that such litigation is filed.
+=head1 COPYRIGHT AND LICENSE
 
-Disclaimer of Warranty: THE PACKAGE IS PROVIDED BY THE COPYRIGHT HOLDER
-AND CONTRIBUTORS "AS IS' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
-THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
-PURPOSE, OR NON-INFRINGEMENT ARE DISCLAIMED TO THE EXTENT PERMITTED BY
-YOUR LOCAL LAW. UNLESS REQUIRED BY LAW, NO COPYRIGHT HOLDER OR
-CONTRIBUTOR WILL BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, OR
-CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THE PACKAGE,
-EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+This software is copyright (c) 2013 - 2022 by Ben Tilly, Rent.com, Thomas Klausner.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-1; # End of LWP::Authen::OAuth2
